/*
  the yaf producer interface for the deviceConfigServer/streamProducer
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */



#include <producer/yaf/yafProducer.h>



static int instance=0;

YafProducer::YafProducer() {
  config=new DeviceConfig();

  musicInfo=config->getMusicInfo();
  yafAudioInfo=config->getAudioInfo();
  statusInfo=config->getStatusInfo();
  audioBuffer=config->getAudioBuffer();
  
  audioBuffer->setPreferredFillGrade(4096);
  control=new YafXPlayerControl();
  stream=new YafStream(this);

  control->addYafListener(this);

  lClearRequest=false;
  pauseCmdNr=-1;
  fullUpdate=false;
  eofBytes=-1;

  instanceCnt=instance;
  instance++;
  notifyBridge=new NotifyMainThread(this);


}


YafProducer::~YafProducer() {
  delete config;
  delete control;
  delete stream;
  delete notifyBridge;
}


void YafProducer::updateDeviceConfig(DeviceConfig* newConfig) {
  AudioBuffer* newAudioBuffer=newConfig->getAudioBuffer();
  YafAudioInfo* newAudioInfo=newConfig->getAudioInfo();
  int bytes;

  if (fullUpdate) {
    config->copyTo(newConfig);
    fullUpdate=false;
  }

  stream->fillBuffer(newAudioBuffer);
  bytes=newAudioBuffer->getReadBytes();
  newAudioInfo->setPlayedBytes(bytes);

  // if we read all bytes we make sure that the next
  // statusmessage is "stopped"
  if ((bytes == eofBytes) && (eofBytes > -1)) {
    cout << "readBytes == eofBytes: "<<eofBytes<<endl;
    eofBytes=-1;
    notifyBridge->sendNotify((char)_STATUS_STOPPED);  // safe signal to gui
  }

}


int YafProducer::open(char* filename ) {
  int back;
  clearStream();
  back=control->open(filename);
  control->send("musicinfo\n");
  return back;
}


int YafProducer::close() {
  control->close();
  clearStream();
  return true;
}


int YafProducer::play() {
  control->play();
  dataUpdate();
  return true;
}


int YafProducer::pause() {
  setThreadState(_STATUS_PAUSED);
  control->pause();
  return true;
}


int YafProducer::jump(int sec) {
  clearStream();
  control->jump(sec);
  play();

  return true;
}


void YafProducer::processRuntimeCommand(int cmd,char* args) {
  int val;
  int point;

  if (cmd == _PLAYER_RUN_CHANNELS) {
    sscanf(args,"%d %d",&val,&point);
    cout << "player channels:"<<val<<endl;
    yafAudioInfo->setStereo(0);
    if (val == 2) {
      yafAudioInfo->setStereo(1);
    }
    dataUpdate();
  }
  if (cmd == _PLAYER_RUN_SAMPLESIZE) {
    sscanf(args,"%d %d",&val,&point);
    yafAudioInfo->setSampleSize(val);
    cout << "player sampleSize:"<<val<<endl;
    dataUpdate();
  }
  if (cmd == _PLAYER_RUN_SPEED) {
    sscanf(args,"%d %d",&val,&point);
    yafAudioInfo->setSpeed(val);
    cout << "player speed:"<<val<<endl;
    dataUpdate();
  }

  if (cmd == _PLAYER_RUN_SONG_NAME) {
    cout << "_PLAYER_SONG_NAME:"<<args<<"***"<<endl;
    musicInfo->setName(args);
    musicInfo->setChange(true);
    dataUpdate();
  }
  if (cmd == _PLAYER_RUN_SONG_LEN) {
    sscanf(args,"%d",&val);
    cout << "_PLAYER_SONG_LEN:"<<val<<"***"<<endl;
    musicInfo->setSec(val);
    musicInfo->setChange(true);
    dataUpdate();
  }


  if (cmd == _PLAYER_RUN_FILEOPEN) {
    if (strcmp(args,"before")==0) {
     cout <<"::::::::::::::::::::::::.fifo Kralle ::::::::::::::"<<endl;
     // start reading fifo
     stream->lockReaderThread();
     stream->openFifo();
     stream->unlockReaderThread();
     control->setOnline(true);
    }
  }
  if (cmd == _PLAYER_RUN_PLAYER_STATUS) {
    if (strlen(args) >= 3) {
      if ( strncmp("off",args,3) == 0 ) {
	sscanf(args,"off %ld",&eofBytes);
	cout << "*********************************eofPos:"<<eofBytes<<endl;
      }
    }
  }
}

void YafProducer::processReturnCommand(int cmdNr,int cmdId,
				       char* ret,char* args) {
  if (cmdNr == pauseCmdNr){
    pauseCmdNr=-1;
    stream->clear();
    stream->setWriteRing(true);
    eofBytes=-1;
    control->setOnline(true);
    lClearRequest=false;
  }

}


void YafProducer::addArgument(char* arg) {
  control->addArgument(arg);
}


void YafProducer::startDecoder() {

  control->startDecoder();
  stream->setWriteRing(false);     // do not write in RingBuffer
  stream->clear();                 // clear RingBuffer
  
  control->send(stream->getFifoCommand());
  control->setOnline(false);

}

void YafProducer::bufferFilledNotify(int filled) {
  audioBuffer->setBufferFilled(filled);
  dataUpdate();
}


void YafProducer::clearStream() {
  if (lClearRequest == false) {
    cout <<"sendStop"<<endl;
    setThreadState(_STATUS_STOPPED);
    lClearRequest=true;
    pauseCmdNr=control->send("pause\n");
    stream->setWriteRing(false);
    stream->clear();
    control->setOnline(false);  // buffer following commands
  }
}




void YafProducer::dataUpdate() {
  int lNotify=true;
  
  if (audioBuffer->getBufferFilled() == false) {
    lNotify=false;
  }
  if (yafAudioInfo->isValid() == false) {
    lNotify=false;
  }
  if (musicInfo->isValid() == false) {
    lNotify=false;
  }

  if (lNotify) {
    setThreadState(_STATUS_PLAYING);
  }
}


void YafProducer::processNotify(char msgId) {
  setThreadState((int)_STATUS_STOPPED);
}


void YafProducer::setThreadState(int state) {
  ThreadState* client;
  client=getNotifyClient();

  client->setProducerEnter(false);
  
  cout << "setThreadState:"<<state<<endl;
  // always store the last (and valid) value in our config
  // this will be copied after the streamProducer
  // has delivered all states.
  statusInfo->setStatus(state);

  
  if (state==_STATUS_PLAYING) {
    client->enqueueThreadState(state);
    client->setProducerEnter(true);
  }
  if (state==_STATUS_PAUSED) {
    client->enqueueThreadState(state);
    client->setProducerEnter(false);
  }  
  if (state==_STATUS_STOPPED) {
    yafAudioInfo->makeInvalid();
    musicInfo->makeInvalid();
    audioBuffer->setBufferFilled(false);

    client->enqueueThreadState(state);
    client->setProducerEnter(false);
  }  
  fullUpdate=true;
}
    


